pub enum CacheDir {
IsEnabled(bool),
CustomPath(PathBuf),
}Expand description
A cache directory config field.
This cache directory configuration field is optional. It defaults to being enabled with the default config path, but also allows a custom path to be set.
Variants
IsEnabled(bool)
Whether the cache directory is enabled with the default path (true),
or disabled (false).
CustomPath(PathBuf)
Enable the cache directory and use a custom path.
Implementations
Sourceimpl CacheDir
impl CacheDir
Sourcepub fn default_path() -> Self
pub fn default_path() -> Self
Returns a CacheDir enabled with the default path.
Sourcepub fn custom_path(path: impl AsRef<Path>) -> Self
pub fn custom_path(path: impl AsRef<Path>) -> Self
Returns a custom CacheDir enabled with path.
Sourcepub fn is_enabled(&self) -> bool
pub fn is_enabled(&self) -> bool
Returns true if this CacheDir is enabled with the default or a custom path.
Sourcepub fn peer_cache_file_path(&self, network: &Network) -> Option<PathBuf>
pub fn peer_cache_file_path(&self, network: &Network) -> Option<PathBuf>
Returns the peer cache file path for network, if enabled.
Sourcepub fn zakura_node_secret_key_file_path(&self, network: &Network) -> PathBuf
pub fn zakura_node_secret_key_file_path(&self, network: &Network) -> PathBuf
Returns the default persistent Zakura iroh secret-key file path for network.
This path is intentionally independent from the peer cache directory so state or cache snapshots do not clone a node’s long-term iroh identity.