pub struct ConnectionInfo {
pub connected_addr: ConnectedAddr,
pub remote: VersionMessage,
pub local: VersionMessage,
pub negotiated_version: Version,
pub is_protected_peer: bool,
}Expand description
The metadata for a peer connection.
Fields
connected_addr: ConnectedAddrThe connected peer address, if known. This address might not be valid for outbound connections.
Peers can be connected via a transient inbound or proxy address, which will appear as the connected address to the OS and Zebra.
remote: VersionMessageThe network protocol VersionMessage sent by the remote peer.
local: VersionMessageThe network protocol VersionMessage sent by this node.
negotiated_version: VersionThe network protocol version negotiated with the remote peer.
Derived from remote.version and the
current zakura_network protocol version.
is_protected_peer: boolWhether this is an inbound connection from an operator-configured protected peer (a block-gossip / zcashd-compat sidecar).
Protected peers follow this node and learn the chain tip only from it,
so they are exempt from the inbound-overload connection drop (their
requests are still shed for backpressure, but the connection is not
severed). See [Connection::handle_inbound_overload].