pub struct HeaderSyncHandle { /* private fields */ }Expand description
Cheap cloneable handle used by other services to inform header sync.
Implementations
Sourceimpl HeaderSyncHandle
impl HeaderSyncHandle
Sourcepub async fn send(
&self,
event: HeaderSyncEvent,
) -> Result<(), SendError<HeaderSyncEvent>>
pub async fn send( &self, event: HeaderSyncEvent, ) -> Result<(), SendError<HeaderSyncEvent>>
Send a fact/event to the header-sync reactor.
Sourcepub fn try_send(
&self,
event: HeaderSyncEvent,
) -> Result<(), TrySendError<HeaderSyncEvent>>
pub fn try_send( &self, event: HeaderSyncEvent, ) -> Result<(), TrySendError<HeaderSyncEvent>>
Try to send a fact/event without awaiting.
Sourcepub fn send_lifecycle(
&self,
event: HeaderSyncEvent,
) -> Result<(), SendError<HeaderSyncEvent>>
pub fn send_lifecycle( &self, event: HeaderSyncEvent, ) -> Result<(), SendError<HeaderSyncEvent>>
Send a peer lifecycle event without sharing the bounded wire-event queue.
Sourcepub fn subscribe_tip(&self) -> Receiver<(Height, Hash)>
pub fn subscribe_tip(&self) -> Receiver<(Height, Hash)>
Subscribe to best-header frontier updates.
Sourcepub fn best_header_tip(&self) -> (Height, Hash)
pub fn best_header_tip(&self) -> (Height, Hash)
Return the currently cached best-header frontier.
Sourcepub fn subscribe_peer_snapshot(&self) -> Receiver<ServicePeerSnapshot>
pub fn subscribe_peer_snapshot(&self) -> Receiver<ServicePeerSnapshot>
Subscribe to header-sync peer slot snapshots.
Sourcepub fn peer_snapshot(&self) -> ServicePeerSnapshot
pub fn peer_snapshot(&self) -> ServicePeerSnapshot
Return the currently cached peer slot snapshot.
Sourcepub fn subscribe_candidate_state(
&self,
) -> Receiver<ZakuraHeaderSyncCandidateState>
pub fn subscribe_candidate_state( &self, ) -> Receiver<ZakuraHeaderSyncCandidateState>
Subscribe to header-sync candidate hints for discovery selection.
Sourcepub fn candidate_state(&self) -> ZakuraHeaderSyncCandidateState
pub fn candidate_state(&self) -> ZakuraHeaderSyncCandidateState
Return the currently cached header-sync candidate hints.